/**
 ** pif_CryptoSuite.ChaCha20
 **   Version: alpha.1.20200102
 **   Release Date: January 2, 2020
 **
 ***************************************************************************************************
 ***************************************************************************************************

pif_CryptoSuite.ChaCha20 is an implementation of the ChaCha20 encryption algorithm---a modification
of the Salsa20 algorithm, both developed by Daniel J. Bernstein. It is a popular stream cipher, with
Google in particular selecting it for use in encrypting internet traffic on Android devices.

 +------------------------------------------+
 |                                          |
 |  Contents                                |
 |  1. License                              |
 |  2. Dependencies                         |
 |  3. Preprocessors                        |
 |  4. Introduction                         |
 |  5. Usage                                |
 |  6. The /pif_CryptoSuite/ChaCha20 class  |
 |  7. Release Notes                        |
 |  8. To Do/Plans                          |
 |                                          |
 +------------------------------------------+

  1. License
  ----------------------------------------------------

  The code for pif_CryptoSuite.ChaCha20 is released under the MIT License. Refer to "+License.dm"
  for the full text of the license.

  2. Dependencies
  ----------------------------------------------------

  This library depends on pif_LongInt to function.

  3. Preprocessors
  ----------------------------------------------------

  By default, the namespace of this library is prefixed, i.e. /pif_CryptoSuite has the prefix _pif
  on it. This can be removed by either,

    1. Defining PIF_NOPREFIX_CRYPTOSUITE, which will remove the prefix from only this library.
    2. Defining PIF_NOPREFIX_GENERAL, which will remove the prefix from this library and all others
       of mine that in the future will have similar prefixing.

  As expected, the unprefixed namespace is /CryptoSuite.

  4. Introduction
  ----------------------------------------------------

  ChaCha20 is an encryption algorithm, used to securely transfer information on insecure or public
  channels. It is used by a number of organizations, most notably Google, and is considered quite
  secure for general use.

  5. Usage
  ----------------------------------------------------

  This implementation of ChaCha20 uses a 256-bit key, a 96-bit nonce, and a 32-bit counter. This
  matches the IETF's implementation in RFC 7539. Note that this is different from the standard
  implementation, A.K.A. Bernstein's original implementation, which uses a 64-bit nonce and a 64-
  bit counter.

  To encrypt a message, one first creates an instance of a /pif_CryptoSuite/ChaCha20 class and sets
  up a key. For the purposes of this library, keys are 256-bit numbers encoded as a hexadecimal
  string. Equivalently, they are 64-character long strings of hex digits. We use a randomly-
  generated key for this example.

    var/pif_CryptoSuite/ChaCha20/Out = new
    Out.SetKey("3905d76e0f1c2414c773b45e65b8887077e5b848463697a8bee06806d45c9f08")

  The key is kept secret, and needs to be securely-shared between the communicating parties in order
  for the encrypted messages to remain secret.

  Additionally, one may optionally set a nonce and a counter. A nonce (in this library) is a 96-bit
  number encoded as a hexadecimal string (equivalently, a string consisting of 24 hexadecimal
  digits). The nonce, unlike the key, is not considered private or secure information. It may be
  shared freely on public channels, and can be specified however one prefers (e.g., it can be a
  counter, a hash from a timestamp, or anything else so long as it's 96 bits and composed of
  hexadecimal digits). As above, we use a randomly generated nonce for this example.

    Out.SetNonce("8f28eade75ad646500c349bf")

  The purpose of a nonce is to prevent certain types of cryptographic attacks that could try and
  determine the secret key, the message sent, or modify the message itself. Every message encrypted
  with a given key should use a different nonce, which is why timestamps and message counters are
  common choices.

  There is also a counter, which is a 32-bit unsigned integer used to change the state of the cipher
  as we encrypt messages. One may manually set this with /pif_CryptoSuite/ChaCha20.SetCounter(), but
  generally one does not want or need to do this. It is useful if you wish to move to a specific
  block without having to compute all the previous blocks.

  To encrypt a message, one passes a plaintext to Out via the Encrypt() method:

    world << Out.Encrypt("Hello, world!") // Prints "2130dbb7c29195b3aefe13a788"

  To decrypt, one does not wish to re-use the Out class for this. Because the counter value has
  increased, generally this will produce useless, nonsensical gobbledygook:

    /*

    Produces the (UTF-8 encoded) string

      "\x5f\x71\x71\x17\xf1\xa2\xbb\xb6\xf1\xa2\xbb\xb6\x39\x6f\x0c"

    which Dream Maker would get upset trying to display. For comparison, the original string
	"Hello, world!" is encodes into UTF-8 as

      "\x48\x65\x6c\x6c\x6f\x2c\x20\x77\x6f\x72\x6c\x64\x21"

    so these are substantially different.

    */
    world << Out.Decrypt("2130dbb7c29195b3aefe13a788")

  Instead, one wants to create a parallel /pif_CryptoSuite/ChaCha20 object with the same key and
  nonce used to encrypt the message. This maintains the same counter between the two.

    var/pif_CryptoSuite/ChaCha20/In = new(
      Key   = "3905d76e0f1c2414c773b45e65b8887077e5b848463697a8bee06806d45c9f08",
      Nonce = "8f28eade75ad646500c349bf"
    )

    world << In.Decrypt("2130dbb7c29195b3aefe13a788") // Prints "Hello, world!"

  6. The /pif_CryptoSuite/ChaCha20 class
  ----------------------------------------------------

    +--------------------------------------+
    |  Parent Class: /pif_CryptoSuite      |
    |  Namespace: /pif_CryptoSuite         |
    |  File: ChaCha20.dm, pif_CryptoSuite  |
    +--------------------------------------+

  The /pif_CryptoSuite/ChaCha20 class is an implementation of the ChaCha20 encryption algorithm.

    New()
    --------------------

      Behavior.

        Instantiates a new /pif_CryptoSuite/ChaCha20 object. One may optionally specify a key, a
		nonce, and/or a counter.

      Arguments.

        - Key (optional, default: null). Refer to SetKey() method.
        - Nonce (optional, default: null). Refer to SetNonce() method.
        - Counter (optional, default: 0). Refer to SetCounter() method.

      Returns.

        1, on success.

    Counter()
    --------------------

      Returns.

        A /pif_LongInt/Unsigned32 object corresponding to the current state of the counter.

    Decrypt()
    --------------------

      Behavior.

        Decrypts a hexadecimal-encoded ciphertext into a Unicode-encoded plaintext.

      Arguments.

        - Ciphertext. A hexadecimal-encoded string, which should be the ciphertext output by the
		  ChaCha20 algorithm.
        - Key (optional, default: null). Changes the Key for the algorithm, if value is different
		  from null. Refer to SetKey() for further details.
        - Nonce (optional, default: null). Changes the nonce for the algorithm, if the value is
		  different from null. Refer to SetNonce() for further details.
        - Counter (optional, default: null) Changes the counter for the algorithm, if the value is
		  different from null.

      Returns.

        A Unicode-encoded (i.e., "plain") string.

    Encrypt()
    --------------------

      Behavior.

        Encrypts a Unicode-encoded plaintext into a hexadecimal-encoded ciphertext.

      Arguments.

        - Plaintext. A string.
        -

        - Key (optional, default: null). Changes the Key for the algorithm, if value is different
		  from null. Refer to SetKey() for further details.
        - Nonce (optional, default: null). Changes the nonce for the algorithm, if the value is
		  different from null. Refer to SetNonce() for further details.
        - Counter (optional, default: null) Changes the counter for the algorithm, if the value is
		  different from null.

      Returns.

        A hexadecimal-encoded string.

    Increment()
    --------------------

      Behavior.

        Increments the counter (i.e., increases it by one).

      Returns.

        1, on success.

    Key()
    --------------------

      Returns.

        The current key, which is a 64-character string of hexadecimal digits.

    Nonce()
    --------------------

      Returns.

        The current nonce, which is a 24-character string of hexadecimal digits.

    SetCounter()
    --------------------

      Behavior.

        Set the counter to the specified value.

        Note: Generally, there should be little need to use this method except perhaps to resync two
		      /ChaCha20 objects which have fallen out of sync. Otherwise, only use if you really
			  know what you're doing.

      Arguments.

        An integer encoded in a GAAF format (refer to "pif_Arithmetic.dm") in pif_LongInt for
		details.

      Returns.

        A /pif_LongInt/Unsigned32 object corresponding to the current counter value.

    SetKey()
    --------------------

    Behavior.

      Sets the key to the specified value.

      Note: The key should be a 64-character string consisting of hexadecimal digits. This is
	        equivalent to the key being a 256-bit integer.

    Arguments.

      A 64-character string of hexadecimal digits.

    Returns.

      1, on success. 0, on failure.

    SetNonce()
    --------------------

    Behavior.

      Sets the nonce to the specified value.

      Note: The nonce should be a 24-character string consisting of hexadecimal digits. This is
	        equivalent to the nonce being a 96-bit integer.

    Arguments.

      A 24-character string of hexadecimal digits.

    Returns.

      1, on success. 0, on failure.

  7. Release Notes
  ----------------------------------------------------

  Version alpha.1.20200102

    - Updated the library so it works with BYOND's new Unicode support.
    - Added documentation.
    - Added a compiler error in the case that pif_LongInt is not present.

    Note: This library's second release is also on its first birthday. Hurrah!

  Version alpha.0.20190102

    - Initial release.

  8. To Do/Plans
  ----------------------------------------------------

  Since alpha.1.20200102

   - Add exceptions and better handling of incorrect or invalid data (e.g., invalid keys, nonces,
     or counters).
   - Make /pif_CryptoSuite/ChaCha20._utf8decode() verify data it is reading in, and output data on
     invalid characters as necessary.

  Since alpha.0.20190102

   - Improve speed (contingent on pif_LongInt, to a significant extent).

 ***************************************************************************************************
 ***************************************************************************************************/